!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine Lyapunov_exponents(numberofdeviationvectors,qnorm,suma,t,LEs)
implicit none
integer(kind=4),intent(in)	:: numberofdeviationvectors
real(kind=8),intent(inout)	:: qnorm(numberofdeviationvectors), t
real(kind=8),intent(out)	:: LEs(numberofdeviationvectors)
real(kind=8),intent(inout)	:: suma(numberofdeviationvectors)
integer(kind=4)			:: i, k

suma(:)=suma(:)+dlog(qnorm(:))
LEs(:)=suma(:)/t

do i=1,numberofdeviationvectors
	do k=1,numberofdeviationvectors-1
		if (LEs(k)<LEs(k+1)) then
			call swap_Lyap_exp(numberofdeviationvectors,LEs,k,k+1)
		endif
	enddo
enddo

return
end subroutine Lyapunov_exponents

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine swap_Lyap_exp(n,sumkt,index_1,index_2)
implicit none
integer(kind=4),intent(in)	:: n, index_1, index_2
real(kind=8),intent(inout)	:: sumkt(n)
real(kind=8)			:: temp

temp=sumkt(index_1)
sumkt(index_1)=sumkt(index_2)
sumkt(index_2)=temp

return
end subroutine swap_Lyap_exp

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine Gram_Schmidt_orthogonormalization(n,numberofdeviationvectors,q,qnorm)
implicit none
integer(kind=4),intent(in)	:: n, numberofdeviationvectors
real(kind=8),intent(inout)	:: q((1+numberofdeviationvectors)*n)
real(kind=8),intent(out)	:: qnorm(numberofdeviationvectors)
real(kind=8)			:: projection((1+numberofdeviationvectors)*n)
integer(kind=4)			:: i, index

index=1
qnorm(index)=dsqrt(dot_product(q(index*n+1:(1+index)*n),q(index*n+1:(1+index)*n)))
q(index*n+1:(1+index)*n)=q(index*n+1:(1+index)*n)/qnorm(index)

do index=2,numberofdeviationvectors
	do i=1,index-1
		call projection_u_v(n,numberofdeviationvectors,q,i,index,projection)
		q(index*n+1:(1+index)*n)=q(index*n+1:(1+index)*n)-projection(index*n+1:(1+index)*n)
	enddo
	qnorm(index)=dsqrt(dot_product(q(index*n+1:(1+index)*n),q(index*n+1:(1+index)*n)))
	q(index*n+1:(1+index)*n)=q(index*n+1:(1+index)*n)/qnorm(index)
enddo

return
end subroutine Gram_Schmidt_orthogonormalization

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine projection_u_v(n,numberofdeviationvectors,q,index1,index2,projection)
implicit none
integer(kind=4),intent(in)	:: n, numberofdeviationvectors
integer(kind=4),intent(in)	:: index1, index2
real(kind=8),intent(in)		:: q((1+numberofdeviationvectors)*n)
real(kind=8),intent(out)  	:: projection((1+numberofdeviationvectors)*n)

projection(index2*n+1:(1+index2)*n)=(dot_product(q(index1*n+1:(1+index1)*n),q(index2*n+1:(1+index2)*n))/dot_product(q(index1*n+1:(1+index1)*n),q(index1*n+1:(1+index1)*n)))*q(index1*n+1:(1+index1)*n)

return
end subroutine projection_u_v

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine write_used_time(numfile_input_data,time_begin)
implicit none
integer(kind=4),intent(in)	:: numfile_input_data
real(kind=4),intent(in)		:: time_begin
real(kind=4)			:: time_end

call cpu_time(time_end)
write(numfile_input_data,*)
write(numfile_input_data,'(/,1x,"Time of operation was",1x,f13.3,1x,"seconds")') time_end-time_begin
write(numfile_input_data,*)
write(numfile_input_data,'(1x,"Programmed by Chris Antonopoulos")')
call flush(numfile_input_data)

write(*,'(/,1x,"Time of operation was",1x,f13.3,1x,"seconds")') time_end-time_begin
write(*,*)
write(*,'(1x,"Programmed by Chris Antonopoulos")')

return
end subroutine write_used_time

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine read_input_parameters(perturbation,t0,tf,tstep,beta,si,ndof,numberofdeviationvectors,numfile_info,numfile_rel_ham_err,numfile_Lyap_exps,filename_input_parameters,numfile_LEs_plot_gnu,numfile_IcLE_and_maxdK_dt_vs_nrg,numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,numfile_ts_dK_dt_vs_K,numfile_ts_K_vs_P,numfile_traj,numfile_cobweb_kinetic_energy,numfile_LEs_network_KP_space,numfile_LEs_Benettin_network,numfile_LEs_Benettin_numerical_diff,numfile_kinetic_nrg_derivative)
implicit none
real(kind=8),intent(out)	:: perturbation, t0, tf, tstep, beta
integer(kind=4),intent(out)	:: si, ndof, numberofdeviationvectors, numfile_info, numfile_rel_ham_err, numfile_Lyap_exps, numfile_LEs_plot_gnu, numfile_IcLE_and_maxdK_dt_vs_nrg, numfile_IcLE_and_maxdK_dt_vs_nrg_gnu, numfile_ts_dK_dt_vs_K, numfile_ts_K_vs_P, numfile_traj, numfile_cobweb_kinetic_energy, numfile_LEs_network_KP_space, numfile_LEs_Benettin_network, numfile_LEs_Benettin_numerical_diff, numfile_kinetic_nrg_derivative
character(*),intent(in)		:: filename_input_parameters

open(1,file=filename_input_parameters,status='old')
read(1,*)
read(1,*) perturbation
read(1,*)
read(1,*)
read(1,*) t0
read(1,*)
read(1,*)
read(1,*) tf
read(1,*)
read(1,*)
read(1,*) tstep
read(1,*)
read(1,*)
read(1,*) si
read(1,*)
read(1,*)
read(1,*) ndof
read(1,*)
read(1,*)
read(1,*) numberofdeviationvectors
read(1,*)
read(1,*)
read(1,*) beta

close(1)

numfile_info=1
numfile_rel_ham_err=2
numfile_Lyap_exps=3
numfile_LEs_plot_gnu=4
numfile_IcLE_and_maxdK_dt_vs_nrg=5
numfile_IcLE_and_maxdK_dt_vs_nrg_gnu=6
numfile_ts_dK_dt_vs_K=7
numfile_ts_K_vs_P=8
numfile_traj=9
numfile_cobweb_kinetic_energy=10
numfile_LEs_network_KP_space=11
numfile_LEs_Benettin_network=12
numfile_LEs_Benettin_numerical_diff=13
numfile_kinetic_nrg_derivative=14

return
end subroutine read_input_parameters

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine LEs_plot_gnu_file(numfile_LEs_plot_gnu,char_NN,char_nrg)
implicit none
integer(kind=4),intent(in)	:: numfile_LEs_plot_gnu
character(*),intent(in)		:: char_NN, char_nrg

open(numfile_LEs_plot_gnu,file='LEs_plot_'//char_NN//'_nrg='//char_nrg//'.gnu',status='replace')
write(numfile_LEs_plot_gnu,'("set output ''LEs_vs_t_",a,"_nrg=",a,".eps''")') char_NN, char_nrg
write(numfile_LEs_plot_gnu,'("set term postscript color enhanced")')
write(numfile_LEs_plot_gnu,'("set size 0.6,0.6")')

write(numfile_LEs_plot_gnu,'("set format x ""10^{%L}""")')
write(numfile_LEs_plot_gnu,'("set format y ""10^{%L}""")')

write(numfile_LEs_plot_gnu,'("set logscale xy")')
write(numfile_LEs_plot_gnu,'("set xlabel ''t''")')
write(numfile_LEs_plot_gnu,'("set ylabel ''LEs''")')

write(numfile_LEs_plot_gnu,'("#set title ""Lyapunov exponents"" font ""Times-Roman,20""")')

write(numfile_LEs_plot_gnu,'("#unset key")')
write(numfile_LEs_plot_gnu,'("#set label 1 ''(a)'' at 3.5e7, 1.1 center")')

write(numfile_LEs_plot_gnu,'("plot [1e1:][] ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:2 w l title ""LE_1"", ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:3 w l title ""LE_2"", ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:4 w l title ""LE_3"", ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:5 w l title ""LE_4"", ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:6 w l title ""LE_5"", ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:7 w l title ""LE_6"", ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:8 w l title ""LE_7"", ''fpu_LE_data_",a,"_nrg=",a,".txt'' u 1:9 w l title ""LE_8"", 1/x w l title ""1/t""")') char_NN, char_nrg, char_NN, char_nrg, char_NN, char_nrg, char_NN, char_nrg, char_NN, char_nrg, char_NN, char_nrg, char_NN, char_nrg, char_NN, char_nrg
close(numfile_LEs_plot_gnu)

return
end subroutine LEs_plot_gnu_file

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine IcLE_and_maxdK_dt_vs_nrg_plot_gnu_file(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu)
implicit none
integer(kind=4),intent(in)	:: numfile_IcLE_and_maxdK_dt_vs_nrg_gnu

open(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,file='IcLE_and_maxdK_dt_vs_nrg_plot.gnu',status='replace')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set output ''IcLE_and_maxdK_dt_vs_nrg.eps''")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set term postscript color enhanced")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set size 0.6,0.6")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("#set format x ""10^{%L}""")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set logscale y")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set xlabel ''Energy''")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set ylabel ''''")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("#set title ""I_c^{LE} and max(dK/dt)"" font ""Times-Roman,20""")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("#unset key")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set key right bottom")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("set key spacing 1.7")')

write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("#set label 1 ''destabilization energy=0.1174'' at 1.37,1e-4 center")')

write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("#set arrow 1 from 0.1174068938024310,1e-9 to 0.1174068938024310,1e1 nohead lt 0")')
write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("#set arrow 2 from 0.15,1e-4 to 0.46,1e-4 lt 0")')

write(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu,'("plot [0:][] ''IcLE_and_maxdK_dt_vs_nrg.out'' u 2:3 w l title ""max(dK/dt)"", ''IcLE_and_maxdK_dt_vs_nrg.out'' u 2:4 w l title ""I_c^{Netw}"", ''IcLE_and_maxdK_dt_vs_nrg.out'' u 2:5 w l title ""H_{KS}^{Netw}""")')

close(numfile_IcLE_and_maxdK_dt_vs_nrg_gnu)

return
end subroutine IcLE_and_maxdK_dt_vs_nrg_plot_gnu_file

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine Gram_Schmidt_orthogonalization_2_vectors(ndim,vector_1,vector_2)
implicit none
integer(kind=4),intent(in)	:: ndim
real(kind=8),intent(inout)	:: vector_1(ndim), vector_2(ndim)
real(kind=8)			:: prod_vector, eucl_dist
integer(kind=4)			:: i

prod_vector=dot_product(vector_1,vector_2)
eucl_dist=dsqrt(dot_product(vector_1,vector_1))
vector_2=vector_2-(prod_vector/(eucl_dist**2))*vector_1

return
end subroutine Gram_Schmidt_orthogonalization_2_vectors
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
